@@ -14,7 +14,7 @@ from .agent_editor import CodeEditor
 
 class Box:
 
-    def __init__(self,x,y, name,ishelper=False,parent=None,parent_widget=None):
+    def __init__(self,x,y, name,ishelper=False,parent=None,parent_widget=None,is_support=False):
 
         self.x = x
         self.y = y
@@ -25,6 +25,7 @@ class Box:
 
         self.name = name #.capitalize()
         self.ishelper = ishelper
+        self.is_support = is_support
         self.parent = parent
         self.n_connections = 0
 
@@ -117,9 +118,10 @@ class Connector:
 
         if a != b:
             self.c = Box(self.a.x + 0.5*dx + u, self.a.y + 0.5*dy + v,name,ishelper=True,parent=self)
-            # self.d = Box(self.a.x + 0.75*dx, self.a.y + 0.75*dy,None,ishelper=True,parent=self)
+            self.d = Box(self.a.x + 0.75*dx, self.a.y + 0.75*dy,name+"<support>",ishelper=True,parent=self,is_support=True) # <- is support handle for better line interpolation
         else:
             self.c = Box(self.a.x + 5*u, self.a.y + 5*u ,name,ishelper=True,parent=self)
+            self.d = Box(self.a.x + 0.75*dx, self.a.y + 0.75*dy,name+"<support>",ishelper=True,parent=self,is_support=True)
 
         self.a.ishelper = False
         self.b.ishelper = False
@@ -177,14 +179,14 @@ class MyDrawWidget(QtWidgets.QWidget):
         self.mousex = 0
         self.mousey = 0
 
-        self.interpol_method = "cubic"
+        self.interpol_method = "quadratic"
         self.mode = "select"
 
         self.zoom_factor = 1.0
 
         self.show()
-    
-    
+
+
     def swap_interpol_style(self):
         if self.interpol_method == "cubic":
             self.interpol_method = "quadratic"
@@ -193,7 +195,7 @@ class MyDrawWidget(QtWidgets.QWidget):
         elif self.interpol_method == "slinear":
             self.interpol_method = "cubic"
         self.parent().update()
-    
+
     def wheelEvent(self,event):
 
         numDegrees = event.angleDelta().y()
@@ -344,17 +346,26 @@ class MyDrawWidget(QtWidgets.QWidget):
         #        return
 
         new_conn = Connector(box1,box2,name,items)
+        print("NEW CONN", name)
 
         if name not in self.old_positions:
             self.old_positions[name] = new_conn.c.x, new_conn.c.y
         else:
             new_conn.c.x = self.old_positions[name][0]
             new_conn.c.y = self.old_positions[name][1]
-            box1.n_connections += 1
-            box2.n_connections += 1
+
+        if name+"<support>" not in self.old_positions:
+            self.old_positions[name+"<support>"] = new_conn.d.x, new_conn.d.y
+        else:
+            new_conn.d.x = self.old_positions[name+"<support>"][0]
+            new_conn.d.y = self.old_positions[name+"<support>"][1]
+
+        box1.n_connections += 1
+        box2.n_connections += 1
 
         self.connectors.append(new_conn)
         self.boxes.append(new_conn.c)
+        self.boxes.append(new_conn.d)
 
         return new_conn
 
@@ -382,6 +393,10 @@ class MyDrawWidget(QtWidgets.QWidget):
                 self.update()
                 self.old_positions[box.name] = (box.x,box.y)
 
+            if box.ishelper and box.name == name+"<support>":
+                box.name = new_name+"<support>"
+                self.update()
+                self.old_positions[box.name] = (box.x,box.y)
 
     def remove_connection(self,name):
         self.update()
@@ -462,8 +477,7 @@ class MyDrawWidget(QtWidgets.QWidget):
             qp.drawLine(points[i][0],points[i][1],points[i+1][0],points[i+1][1])
 
 
-
-    def curved_connector(self,qp,x0,y0,xf,yf,xi,yi,dashed=False):
+    def curved_connector(self,qp,x0,y0,xf,yf,xi,yi,xd,yd,dashed=False):
         """
         draws a curved connector
         """
@@ -476,6 +490,7 @@ class MyDrawWidget(QtWidgets.QWidget):
         yi2 = 0.5*(yi-y0) + y0
         yi3 = 0.5*(yf-yi) + yi
 
+
         #qp.drawRect(xi2-5,yi2-5,10,10)
         #qp.drawRect(xi3-5,yi3-5,10,10)
         W = 25 # 25 # 12.5
@@ -586,15 +601,17 @@ class MyDrawWidget(QtWidgets.QWidget):
         end_is_start = False
         if abs(x0-xf) < 0.1 and abs(y0-yf) < 0.1:
             end_is_start = True
-            points2 = np.array([[x0,xi6,xi62,xi2,xi],
-                                [y0,yi6,yi62,yi2,yi]]).T
+            points2 = np.array([[x0,xi62,xi2,xi,xd],
+                                [y0,yi62,yi2,yi,yd]]).T
         else:
-            points2 = np.array([[x0,xi6,xi62,xi2,xi,xi3,xi35,xi5,xf],
-                          [y0,yi6,yi62,yi2,yi,yi3,yi35,yi5,yf]]).T
+            #points2 = np.array([[x0,xi6,xi62,xi2,xi,xd,xi3,xi35,xi5,xf],
+            #              [y0,yi6,yi62,yi2,yi,yd,yi3,yi35,yi5,yf]]).T
 
-            points2 = np.array([[x0,xi2,xi,xi3,xf],
-                          [y0,yi2,yi,yi3,yf]]).T
+            #points2 = np.array([[x0,xi2,xi,xd,xi3,xf],
+            #                      [y0,yi2,yi,yd,yi3,yf]]).T
 
+            points2 = np.array([[x0,xi2,xi,xd,xf],
+                                 [y0,yi2,yi,yd,yf]]).T
 
         points = []
         points.append(points2[0]) # make sure it always starts at x0,y0 and ends at xf, yf
@@ -618,7 +635,7 @@ class MyDrawWidget(QtWidgets.QWidget):
                 method = "quadratic"
                 alpha = np.linspace(0, 1, 50)
             else:
-                method  = self.interpol_method 
+                method  = self.interpol_method
                 alpha = np.linspace(0, 1, 80)
 
             #for method in interpolations_methods:
@@ -915,7 +932,13 @@ class MyDrawWidget(QtWidgets.QWidget):
                 # draw only if filter applies
 
                 offset_box = 25
-                tx,ty = self.curved_connector(qp,self.gox + int(conn.a.x+offset_box), self.goy + int(conn.a.y+offset_box),self.gox + int(conn.b.x+offset_box), self.goy + int(conn.b.y+offset_box), self.gox + int(conn.c.x+5), self.goy + int(conn.c.y+5))
+                tx,ty = self.curved_connector(qp,self.gox + int(conn.a.x+offset_box),
+                                                 self.goy + int(conn.a.y+offset_box),
+                                                 self.gox + int(conn.b.x+offset_box),
+                                                 self.goy + int(conn.b.y+offset_box),
+                                                 self.gox + int(conn.c.x+5), self.goy + int(conn.c.y+5),
+                                                 self.gox + int(conn.d.x+5), self.goy + int(conn.d.y+5),
+                                              )
                 # self.curved_connector(qp,int(conn.a.x), int(conn.a.y),int(conn.b.x), int(conn.b.y),int(conn.c.x), int(conn.c.y))
                 if conn.a not in active_boxes:
                     active_boxes.append(conn.a)
@@ -923,7 +946,8 @@ class MyDrawWidget(QtWidgets.QWidget):
                     active_boxes.append(conn.b)
                 if conn.c not in active_boxes:
                     active_boxes.append(conn.c)
-
+                if conn.d not in active_boxes:
+                    active_boxes.append(conn.d)
 
                 if conn.c == self.highlighted:
                     pen = QtGui.QPen(self.parent().theme_manager.get_color(2),1)
@@ -941,10 +965,24 @@ class MyDrawWidget(QtWidgets.QWidget):
 
                         a = self.zoom_factor*(self.gox +conn.c.x)
                         b = self.zoom_factor*(self.goy + conn.c.y-7)
-                        if self.parent().checkBox_4.isChecked():
-                            qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a-.1),b-10),QtCore.QPoint(int(a+5.1*len(conn.c.name)),b+10)),brush)
 
-                        qp.drawText(a,b,conn.c.name)
+                        # draw second connector
+                        a2 = self.zoom_factor*(self.gox +conn.d.x)
+                        b2 = self.zoom_factor*(self.goy + conn.d.y-7)
+
+
+                        if self.parent().checkBox_4.isChecked():
+                            fm = qp.fontMetrics()
+                            text_length = fm.boundingRect(conn.c.name.replace("_"," ")).width()
+                            # text_length = 5.1*len(conn.c.name)
+                            qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a-.1),b-10),QtCore.QPoint(int(a+text_length),b+7)),brush)
+                            # qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a2-.1),b2-10),QtCore.QPoint(int(a2+5.1*len(conn.d.name)),b2+10)),brush)
+
+                        qp.drawText(a,b,conn.c.name.replace("_"," "))
+
+
+
+
                 else:
                     if self.parent().checkBox_2.isChecked():
                         if self.highlighted == conn.c:
@@ -954,16 +992,42 @@ class MyDrawWidget(QtWidgets.QWidget):
                         if conn.a.x != conn.b.x and conn.a.y != conn.b.y:
                             a = self.zoom_factor*(tx)
                             b = self.zoom_factor*(ty)
+
+                            a2 = self.zoom_factor*(tx+5)
+                            b2 = self.zoom_factor*(ty+5)
+
+
                         else:
                             a = self.zoom_factor*(self.gox +conn.c.x)
                             b = self.zoom_factor*(self.goy + conn.c.y-7)
+
+                            a2 = self.zoom_factor*(self.gox +conn.d.x)
+                            b2 = self.zoom_factor*(self.goy + conn.d.y-7)
+
+
                         if self.parent().checkBox_4.isChecked():
-                            qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a-.1),b-10),QtCore.QPoint(int(a+5.1*len(conn.c.name)),b+10)),brush)
+                            # text_length = 5.1*len(conn.c.name)
+                            fm = qp.fontMetrics()
+                            text_length = fm.boundingRect(conn.c.name.replace("_"," ")).width()
+                            qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a-.1),b-10),QtCore.QPoint(int(a+text_length),b+7)),brush)
+                            # qp.fillRect(QtCore.QRect(QtCore.QPoint(int(a2-.1),b2-10),QtCore.QPoint(int(a2+5.1*len(conn.d.name)),b2+10)),brush)
+
+
                         qp.drawText(a,b,conn.c.name.replace("_"," "))
 
+                        # draw second connector
+
 
             else:
-                self.curved_connector(qp,self.gox + int(conn.a.x+12.5), self.goy + int(conn.a.y+12.5),self.gox + int(conn.b.x+12.5), self.goy + int(conn.b.y+12.5), self.gox + int(conn.c.x+5), self.goy + int(conn.c.y+5),dashed=True)
+                self.curved_connector(qp,self.gox + int(conn.a.x+12.5),
+                 self.goy + int(conn.a.y+12.5),
+                 self.gox + int(conn.b.x+12.5),
+                 self.goy + int(conn.b.y+12.5),
+                 self.gox + int(conn.c.x+5),
+                 self.goy + int(conn.c.y+5),
+                 self.gox + int(conn.d.x+5),
+                 self.goy + int(conn.d.y+5),
+                 dashed=True)
 
         """
         if self.darkMode:
@@ -1029,7 +1093,12 @@ class MyDrawWidget(QtWidgets.QWidget):
 
             if box.ishelper:
                 pen = QtGui.QPen(self.parent().theme_manager.get_color(6),1)
-                br = QtGui.QBrush(self.parent().theme_manager.get_color(1),1)
+
+                if box.is_support:
+                    br = QtGui.QBrush(self.parent().theme_manager.get_color(7),1)
+                else:
+                    br = QtGui.QBrush(self.parent().theme_manager.get_color(1),1)
+
                 qp.setPen(pen)
                 # qp.drawText(x,y-7,box.name)
             else:
@@ -1050,13 +1119,23 @@ class MyDrawWidget(QtWidgets.QWidget):
                 qp.setPen(pen)
                 # qp.drawRect(QtCore.QRect(QtCore.QPoint(x,y),QtCore.QPoint(x2,y2)))
 
+                if box.is_support:
+                    show_box = self.parent().checkBox_6.isChecked()
+                else:
+                    show_box = self.parent().checkBox_3.isChecked()
 
-                if self.parent().checkBox_3.isChecked():
+                if show_box:
                     if box in active_boxes:
-                        qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),6, 6)
-                    else:
-                        qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),2, 2)
+                        if box.is_support:
+                            qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),6, 6)
+                        else:
+                            qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),6, 6)
 
+                    else:
+                        if box.is_support:
+                            qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),2, 2)
+                        else:
+                            qp.drawEllipse(QtCore.QPoint(self.zoom_factor*(self.gox + x+4),self.zoom_factor*(self.goy + y+4)),2, 2)
 
 
                 """
@@ -1199,7 +1278,7 @@ class MyDrawWidget(QtWidgets.QWidget):
 
                     if box.ishelper and box.parent is not None:
                         self.boxes.remove(box.parent.c)
-                        # self.boxes.remove(box.parent.d)
+                        self.boxes.remove(box.parent.d)
                         self.connectors.remove(box.parent)
                         self.update()
                         self.mode = "drag"
@@ -1212,6 +1291,7 @@ class MyDrawWidget(QtWidgets.QWidget):
                                 self.boxes.remove(conn.a)
                                 self.boxes.remove(conn.b)
                                 self.boxes.remove(conn.c)
+                                self.boxes.remove(conn.d)
 
                         if box in self.boxes:
                             self.boxes.remove(box)
@@ -1233,7 +1313,7 @@ class MyDrawWidget(QtWidgets.QWidget):
                         con = Connector(self.connected1,self.connected2)
                         self.connectors.append(con)
                         self.boxes.append(con.c)
-                        # self.boxes.append(con.d)
+                        self.boxes.append(con.d)
 
                         self.connected1 = None
                         self.connected2 = None
