@@ -102,7 +102,7 @@ def matplotlib_barplot(data, xlabel, ylabel, title, color="indigo", size=(6, 5),
             ylabel = "Data"
         data = pd.DataFrame({ylabel: data})
         data.set_index(ylabel)
-    
+
     # plt.figure(figsize=size)
     data.plot.bar(color=color, stacked=stacked,ax=plt.gca())
     plt.gca().set_title(title)
@@ -122,13 +122,21 @@ def matplotlib_barplot(data, xlabel, ylabel, title, color="indigo", size=(6, 5),
     if tight_layout:
         plt.tight_layout()
 
+    # fix reverse labeling
+    # https://stackoverflow.com/questions/46908085/bar-plot-does-not-respect-order-of-the-legend-text-in-matplotlib
+    #
+    ax = plt.gca()
+    h, l = ax.get_legend_handles_labels()
+    if legend != "off":
+        ax.legend(h[::-1], l[::-1],loc=legend,framealpha=0.0)
+
     if show:
         plt.show()
     else:
         return plt.gcf()
 
 
-def matplotlib_lineplot(data, xlabel=None, ylabel=None, title="", xlim=None, ylim=None, color="indigo", legend="best",show=False):
+def matplotlib_lineplot(data, xlabel=None, ylabel=None, title="", xlim=None, ylim=None, color="indigo", legend="best",marker=None,show=False):
     """
     creates a line plot in matplotlib. The 'macro' plotting style will be used if it is set up
 
@@ -152,22 +160,24 @@ def matplotlib_lineplot(data, xlabel=None, ylabel=None, title="", xlim=None, yli
         data = pd.DataFrame({ylabel: data})
         data.set_index(ylabel)
 
-    data.plot(color=color)
+    for col in data.columns:
+        plt.plot(data[col],color=color,marker=marker,label=col)
     plt.xlim(xlim)
     plt.ylim(ylim)
     plt.title(title)
 
-    if legend != "off" and legend == "outside":
+    if legend != "off":
         plt.legend(loc=legend)
     elif legend == "outside":
         plt.legend(loc=(1.02,0))
 
-    else:
+    elif legen == "off":
         old_legend = plt.gca().get_legend()
         if old_legend is not None:
             old_legend.remove()
-        if ylabel is not None:
-            plt.ylabel(ylabel)
+
+    if ylabel is not None:
+        plt.ylabel(ylabel)
 
     if xlabel is not None:
         plt.gca().set_xlabel(xlabel)
@@ -335,7 +345,7 @@ def draw_band(fig,pstart:Point, pend:Point, color="blue",alpha=.8,weight=2,norm=
             plt.gca().text(bx,by, "%.2f" %(weight/norm), props, rotation=rot,color="black",alpha=alpha)
         # plt.annotate(str(weight),(bx,by),color="gray",alpha=alpha)
 
-def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
+def plot_sankey(data,title="",show_plot=True,show_values=True,colors=None):
     """
     plots a sankey diagram from data
 
@@ -343,6 +353,7 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
     :param title: title of the plot
     :param show_plot: boolean switch to show plot window (default True). If False, figure is returned
     :param show_values: boolean switch to print numerical values of data as text label (default True)
+    :param colors: (optional) a list of colors (if None, default colors are chosen)
 
     :return fig: None or matplotlib figure
 
@@ -373,19 +384,20 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
 
 
     # TODO custom colors or custom color themes
-    colors = [
-        "black",
-        "red",
-        "blue",
-        "gold",
-        "dimgray",
-    ]
+    if colors is None:
+        colors = [
+            "black",
+            "red",
+            "blue",
+            "gold",
+            "dimgray",
+        ]
 
 
 
     # 1. normalize the values to reasonable width of bands
 
-    norm_factor = 1.0 #normalization factor for with
+    norm_factor = 1.5 #normalization factor for with
 
     maxval = 0.0
     maxrows = 2
@@ -397,21 +409,21 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
 
             maxval = max(val,maxval)
 
-    norm_factor = 1.0
+    norm_factor = 1.5
     if maxval > 0.0:
         norm_factor = 1.0 /maxval
 
-    fig = plt.figure(figsize=(4.6*len(data),1.8*maxrows))
+    fig = plt.figure(figsize=(4.6*len(data),1.4*maxrows)) # 1.8*maxrows
 
     # 2. plot the bezier patches
 
     i = 10  #
-    j0 = 10 # < starting position (i,j)
+    j0 = 6 # < starting position (i,j)
 
-    dy = 8  # vertical distance of bands
-    dx = 40  # horizontal distance of layers
+    dy = 10 # 8  # vertical distance of bands
+    dx = 55 #   # horizontal distance of layers
 
-    dx_space = 2 # white space between layers
+    dx_space = 2.5 # white space between layers
 
     # define some points for the unique nodes
     my_points = {} # keys will be names, value will be Points
@@ -431,7 +443,7 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
                 my_points[name] = new_point
 
             if name not in my_labels:
-                my_labels[name] = Label(new_point.x-2.2,new_point.y-.2,name) # plt.annotate(name,(new_point.x-1.8, new_point.y-.1))
+                my_labels[name] = Label(new_point.x-5.2,new_point.y-.2,name) # plt.annotate(name,(new_point.x-1.8, new_point.y-.1))
             else:
 
                 my_labels[name].shift_up()
@@ -441,6 +453,7 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
         j = j0
         for name in layer["to"].unique():
 
+
             if name not in my_points:
                 new_point = Point(i+dx,j)
                 my_points[name] = new_point #Point(i+dx,j)
@@ -467,7 +480,7 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
 
             fromnode = my_points[row["from"]]
             tonode = my_points[row["to"]]
-            weight = max(0.1,float(row["value"])*norm_factor)
+            weight = max(0.5,float(row["value"])*norm_factor) # <- make bars thicker
 
             draw_band(fig,fromnode,tonode,colors[color_idx%len(colors)],weight=weight,norm=norm_factor,show_label=show_values,label=str(row["value"]))
 
@@ -475,9 +488,9 @@ def plot_sankey(data,title="Sankey Diagram",show_plot=True,show_values=True):
 
     for label in my_labels.values(): # draw the labels here...
         if len(label.text) <= 2:
-            plt.annotate(label.text,(label.x,label.y))
+            plt.annotate(label.text.replace("$[source]","").replace("$[sink]",""),(label.x,label.y))
         else: # rotate label for better visibility
-            plt.gca().text(label.x,label.y, label.text, rotation=45)
+            plt.gca().text(label.x,label.y, label.text.replace("$[source]","").replace("$[sink]",""), rotation=60,fontsize=7)
 
     # 3. return the figure or show window
 
